"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalCodeGenerator = void 0;
const CodeGenConfig_1 = require("./CodeGenConfig");
const CodeWriter_1 = require("./CodeWriter");
class GenerateClassConfig {
    namespaceName;
    exportCodePath;
    pkgId;
    pkgName;
    getMemberByName;
    classNamePrefix;
    checkCustomClass;
}
class NormalCodeGenerator {
    classes;
    getCustomClassName(className) {
        return CodeGenConfig_1.CodeGenConfig.ClassNamePrefix + className;
    }
    Handle(handler) {
        let settings = handler.project.GetSettings("Publish").codeGeneration;
        let codePkgName = handler.ToFilename(handler.pkg.name); //convert chinese to pinyin, remove special chars etc.
        let exportCodePath = "../../Unity/Assets/Mono/PeachValley/FUI/AutoGeneratedCode/" + codePkgName;
        //let exportCodePath = handler.exportCodePath + '/' + codePkgName;
        let namespaceName = CodeGenConfig_1.CodeGenConfig.NameSpacePrefix + codePkgName;
        if (settings.packageName)
            namespaceName = settings.packageName + '.' + namespaceName;
        let classes = handler.CollectClasses(settings.ignoreNoname, settings.ignoreNoname, null);
        this.classes = classes;
        //handler.SetupCodeFolder(exportCodePath, "cs"); //check if target folder exists, and delete old files
        handler.SetupCodeFolder(exportCodePath, ".cs");
        let getMemberByName = settings.getMemberByName;
        let classCnt = classes.Count;
        let writer = new CodeWriter_1.default();
        for (let i = 0; i < classCnt; i++) {
            let classInfo = classes.get_Item(i);
            this.GenerateComponentClass(classInfo, writer, {
                namespaceName: namespaceName,
                exportCodePath: exportCodePath,
                pkgId: handler.pkg.id,
                pkgName: handler.pkg.name,
                getMemberByName: getMemberByName,
                classNamePrefix: CodeGenConfig_1.CodeGenConfig.ClassNamePrefix,
                checkCustomClass: this.isCustomClass.bind(this),
            });
        }
        writer.reset();
        let binderName = CodeGenConfig_1.CodeGenConfig.ClassNamePrefix + codePkgName + 'Binder';
        writer.writeln('using FairyGUI;');
        writer.writeln();
        writer.writeln('namespace %s', namespaceName);
        writer.startBlock();
        writer.writeln('public class %s', binderName);
        writer.startBlock();
        writer.writeln('public static void BindAll()');
        writer.startBlock();
        for (let i = 0; i < classCnt; i++) {
            let classInfo = classes.get_Item(i);
            let className = this.getCustomClassName(classInfo.className);
            writer.writeln('UIObjectFactory.SetPackageItemExtension(%s.URL, typeof(%s));', className, className);
        }
        writer.endBlock(); //bindall
        writer.endBlock(); //class
        writer.endBlock(); //namespace
        writer.save(exportCodePath + '/' + binderName + '.cs');
    }
    // 判断是不是我们自定义类型组件
    isCustomClass(typeName) {
        let classes = this.classes;
        let count = this.classes.Count;
        for (let k = 0; k < count; k++) {
            if (typeName === classes.get_Item(k).className) {
                return true;
            }
        }
        return false;
    }
    GenerateComponentClass(classInfo, writer, config) {
        let getMemberByName = config.getMemberByName;
        let namespaceName = config.namespaceName;
        let classNamePrefix = config.classNamePrefix ?? "";
        let className = classNamePrefix + classInfo.className;
        let members = classInfo.members;
        writer.reset();
        writer.writeln('using FairyGUI;');
        writer.writeln('using FairyGUI.Utils;');
        writer.writeln();
        writer.writeln('namespace %s', namespaceName);
        writer.startBlock();
        writer.writeln('public partial class %s : %s', className, classInfo.superClassName);
        writer.startBlock();
        writer.writeln(`public static string UIPackageName = "%s";
        public static string UIResName = "%s";`, config.pkgName, classInfo.resName);
        let memberCnt = members.Count;
        for (let j = 0; j < memberCnt; j++) {
            let memberInfo = members.get_Item(j);
            let memberType = memberInfo.type;
            let typeName = config.checkCustomClass(memberType) ? config.classNamePrefix + memberType : memberType;
            writer.writeln('public %s %s;', typeName, memberInfo.varName);
        }
        writer.writeln('public const string URL = "ui://%s%s";', config.pkgId, classInfo.resId);
        writer.writeln();
        writer.writeln('public static %s CreateInstance()', className);
        writer.startBlock();
        writer.writeln('return (%s)UIPackage.CreateObject("%s", "%s");', className, config.pkgName, classInfo.resName);
        writer.endBlock();
        writer.writeln();
        writer.writeln('public override void ConstructFromXML(XML xml)');
        writer.startBlock();
        writer.writeln('base.ConstructFromXML(xml);');
        writer.writeln();
        for (let j = 0; j < memberCnt; j++) {
            let memberInfo = members.get_Item(j);
            if (memberInfo.group == 0) {
                let memberType = memberInfo.type;
                let typeName = config.checkCustomClass(memberType) ? config.classNamePrefix + memberType : memberType;
                if (getMemberByName)
                    writer.writeln('%s = (%s)GetChild("%s");', memberInfo.varName, typeName, memberInfo.name);
                else
                    writer.writeln('%s = (%s)GetChildAt(%s);', memberInfo.varName, typeName, memberInfo.index);
            }
            else if (memberInfo.group == 1) {
                if (getMemberByName)
                    writer.writeln('%s = GetController("%s");', memberInfo.varName, memberInfo.name);
                else
                    writer.writeln('%s = GetControllerAt(%s);', memberInfo.varName, memberInfo.index);
            }
            else {
                if (getMemberByName)
                    writer.writeln('%s = GetTransition("%s");', memberInfo.varName, memberInfo.name);
                else
                    writer.writeln('%s = GetTransitionAt(%s);', memberInfo.varName, memberInfo.index);
            }
        }
        writer.endBlock();
        writer.endBlock(); //class
        writer.endBlock(); //namespace
        writer.save(config.exportCodePath + '/' + className + '.cs');
    }
}
const normalCodeGenerator = new NormalCodeGenerator();
exports.normalCodeGenerator = normalCodeGenerator;
