using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class ActionFlowConfigCategory : ProtoObject, IMerge
    {
        public static ActionFlowConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, ActionFlowConfig> dict = new Dictionary<int, ActionFlowConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<ActionFlowConfig> list = new List<ActionFlowConfig>();
		
        public ActionFlowConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            ActionFlowConfigCategory s = o as ActionFlowConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (ActionFlowConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public ActionFlowConfig Get(int id)
        {
            this.dict.TryGetValue(id, out ActionFlowConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (ActionFlowConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, ActionFlowConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<ActionFlowConfig> GetList()
        {
            return this.list;
        }

        public ActionFlowConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class ActionFlowConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>类型</summary>
		[ProtoMember(2)]
		public int Type { get; set; }
		/// <summary>下一动作组</summary>
		[ProtoMember(3)]
		public int[] NextActionGroup { get; set; }
		/// <summary>权重</summary>
		[ProtoMember(4)]
		public int[] Weight { get; set; }

	}
}
