using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class AttributesConfigCategory : ProtoObject, IMerge
    {
        public static AttributesConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, AttributesConfig> dict = new Dictionary<int, AttributesConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<AttributesConfig> list = new List<AttributesConfig>();
		
        public AttributesConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            AttributesConfigCategory s = o as AttributesConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (AttributesConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public AttributesConfig Get(int id)
        {
            this.dict.TryGetValue(id, out AttributesConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (AttributesConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, AttributesConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<AttributesConfig> GetList()
        {
            return this.list;
        }

        public AttributesConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class AttributesConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>属性名称</summary>
		[ProtoMember(2)]
		public string Name { get; set; }
		/// <summary>属性类型</summary>
		[ProtoMember(3)]
		public int[] Type { get; set; }
		/// <summary>英文字段</summary>
		[ProtoMember(4)]
		public string Field { get; set; }
		/// <summary>值范围</summary>
		[ProtoMember(5)]
		public long[] Ranges { get; set; }
		/// <summary>属性含义</summary>
		[ProtoMember(6)]
		public string Meaning { get; set; }

	}
}
