using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class BehaviourTypeConfigCategory : ProtoObject, IMerge
    {
        public static BehaviourTypeConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, BehaviourTypeConfig> dict = new Dictionary<int, BehaviourTypeConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<BehaviourTypeConfig> list = new List<BehaviourTypeConfig>();
		
        public BehaviourTypeConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            BehaviourTypeConfigCategory s = o as BehaviourTypeConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (BehaviourTypeConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public BehaviourTypeConfig Get(int id)
        {
            this.dict.TryGetValue(id, out BehaviourTypeConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (BehaviourTypeConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, BehaviourTypeConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<BehaviourTypeConfig> GetList()
        {
            return this.list;
        }

        public BehaviourTypeConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class BehaviourTypeConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>行为类型名称</summary>
		[ProtoMember(2)]
		public string BehaviourName { get; set; }
		/// <summary>事件触发概率</summary>
		[ProtoMember(3)]
		public int TriggerProbability { get; set; }
		/// <summary>叠加概率</summary>
		[ProtoMember(4)]
		public int SuperpositionProbability { get; set; }

	}
}
