using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class ClothesConfigCategory : ProtoObject, IMerge
    {
        public static ClothesConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, ClothesConfig> dict = new Dictionary<int, ClothesConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<ClothesConfig> list = new List<ClothesConfig>();
		
        public ClothesConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            ClothesConfigCategory s = o as ClothesConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (ClothesConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public ClothesConfig Get(int id)
        {
            this.dict.TryGetValue(id, out ClothesConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (ClothesConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, ClothesConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<ClothesConfig> GetList()
        {
            return this.list;
        }

        public ClothesConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class ClothesConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>衣服名称</summary>
		[ProtoMember(2)]
		public string Name { get; set; }
		/// <summary>使用对象</summary>
		[ProtoMember(3)]
		public int[] UsingObjects { get; set; }
		/// <summary>所属季节</summary>
		[ProtoMember(4)]
		public int[] CureMax { get; set; }
		/// <summary>防御效果</summary>
		[ProtoMember(5)]
		public int Efficacy { get; set; }

	}
}
