using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class CuisineConfigCategory : ProtoObject, IMerge
    {
        public static CuisineConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, CuisineConfig> dict = new Dictionary<int, CuisineConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<CuisineConfig> list = new List<CuisineConfig>();
		
        public CuisineConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            CuisineConfigCategory s = o as CuisineConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (CuisineConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public CuisineConfig Get(int id)
        {
            this.dict.TryGetValue(id, out CuisineConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (CuisineConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, CuisineConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<CuisineConfig> GetList()
        {
            return this.list;
        }

        public CuisineConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class CuisineConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>类型</summary>
		[ProtoMember(2)]
		public int Type { get; set; }
		/// <summary>名称</summary>
		[ProtoMember(3)]
		public string Name { get; set; }
		/// <summary>效果</summary>
		[ProtoMember(4)]
		public int Effection { get; set; }
		/// <summary>效果数值</summary>
		[ProtoMember(5)]
		public int EffectionValue { get; set; }
		/// <summary>营养值</summary>
		[ProtoMember(6)]
		public int NutritiveValue { get; set; }
		/// <summary>可转化食物数量</summary>
		[ProtoMember(7)]
		public int FoodNum { get; set; }

	}
}
