using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class DialogueGroupCategory : ProtoObject, IMerge
    {
        public static DialogueGroupCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, DialogueGroup> dict = new Dictionary<int, DialogueGroup>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<DialogueGroup> list = new List<DialogueGroup>();
		
        public DialogueGroupCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            DialogueGroupCategory s = o as DialogueGroupCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (DialogueGroup config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public DialogueGroup Get(int id)
        {
            this.dict.TryGetValue(id, out DialogueGroup item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (DialogueGroup)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, DialogueGroup> GetAll()
        {
            return this.dict;
        }
        
        public List<DialogueGroup> GetList()
        {
            return this.list;
        }

        public DialogueGroup GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class DialogueGroup: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>对白编号</summary>
		[ProtoMember(2)]
		public int[] DialogueId { get; set; }
		/// <summary>对白人数</summary>
		[ProtoMember(3)]
		public int DialogueNumber { get; set; }
		/// <summary>对白顺序</summary>
		[ProtoMember(4)]
		public int[] DialogueOrder { get; set; }

	}
}
