using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class DropGroupConfigCategory : ProtoObject, IMerge
    {
        public static DropGroupConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, DropGroupConfig> dict = new Dictionary<int, DropGroupConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<DropGroupConfig> list = new List<DropGroupConfig>();
		
        public DropGroupConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            DropGroupConfigCategory s = o as DropGroupConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (DropGroupConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public DropGroupConfig Get(int id)
        {
            this.dict.TryGetValue(id, out DropGroupConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (DropGroupConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, DropGroupConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<DropGroupConfig> GetList()
        {
            return this.list;
        }

        public DropGroupConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class DropGroupConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>物品编号</summary>
		[ProtoMember(2)]
		public int[] ItemId { get; set; }
		/// <summary>掉量区间</summary>
		[ProtoMember(3)]
		public int[] DropAmount { get; set; }
		/// <summary>权重</summary>
		[ProtoMember(4)]
		public int[] Weight { get; set; }
		/// <summary>掉量描述</summary>
		[ProtoMember(5)]
		public int[] DropDescription { get; set; }
		/// <summary>关联效果</summary>
		[ProtoMember(6)]
		public int[] RelatedBuff { get; set; }

	}
}
