using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class EventConfigCategory : ProtoObject, IMerge
    {
        public static EventConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, EventConfig> dict = new Dictionary<int, EventConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<EventConfig> list = new List<EventConfig>();
		
        public EventConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            EventConfigCategory s = o as EventConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (EventConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public EventConfig Get(int id)
        {
            this.dict.TryGetValue(id, out EventConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (EventConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, EventConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<EventConfig> GetList()
        {
            return this.list;
        }

        public EventConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class EventConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>类型</summary>
		[ProtoMember(2)]
		public int Type { get; set; }
		/// <summary>事件标题</summary>
		[ProtoMember(3)]
		public string EventName { get; set; }
		/// <summary>掉落物品组</summary>
		[ProtoMember(4)]
		public int[] DropGroup { get; set; }
		/// <summary>事件参数</summary>
		[ProtoMember(5)]
		public int EventParameter { get; set; }
		/// <summary>事件参数值</summary>
		[ProtoMember(6)]
		public int EventParameterValue { get; set; }
		/// <summary>事件简述</summary>
		[ProtoMember(7)]
		public int ShortDescription { get; set; }
		/// <summary>事件描述</summary>
		[ProtoMember(8)]
		public string Description { get; set; }
		/// <summary>对白</summary>
		[ProtoMember(9)]
		public int Dialogue { get; set; }

	}
}
