using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class GameStageConfigCategory : ProtoObject, IMerge
    {
        public static GameStageConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, GameStageConfig> dict = new Dictionary<int, GameStageConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<GameStageConfig> list = new List<GameStageConfig>();
		
        public GameStageConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            GameStageConfigCategory s = o as GameStageConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (GameStageConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public GameStageConfig Get(int id)
        {
            this.dict.TryGetValue(id, out GameStageConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (GameStageConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, GameStageConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<GameStageConfig> GetList()
        {
            return this.list;
        }

        public GameStageConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class GameStageConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>起始天数</summary>
		[ProtoMember(2)]
		public int StartDay { get; set; }
		/// <summary>终止天数</summary>
		[ProtoMember(3)]
		public int EndDay { get; set; }
		/// <summary>开始季节</summary>
		[ProtoMember(4)]
		public int StartSeason { get; set; }

	}
}
