using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class PhysiqueConfigCategory : ProtoObject, IMerge
    {
        public static PhysiqueConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, PhysiqueConfig> dict = new Dictionary<int, PhysiqueConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<PhysiqueConfig> list = new List<PhysiqueConfig>();
		
        public PhysiqueConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            PhysiqueConfigCategory s = o as PhysiqueConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (PhysiqueConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public PhysiqueConfig Get(int id)
        {
            this.dict.TryGetValue(id, out PhysiqueConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (PhysiqueConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, PhysiqueConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<PhysiqueConfig> GetList()
        {
            return this.list;
        }

        public PhysiqueConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class PhysiqueConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>体质等级</summary>
		[ProtoMember(2)]
		public int Lv { get; set; }
		/// <summary>体质经验</summary>
		[ProtoMember(3)]
		public int Exp { get; set; }
		/// <summary>体质段</summary>
		[ProtoMember(4)]
		public int Section { get; set; }
		/// <summary>最低寿命</summary>
		[ProtoMember(5)]
		public int LifeMin { get; set; }
		/// <summary>最高寿命</summary>
		[ProtoMember(6)]
		public int LifeMax { get; set; }
		/// <summary>劳力经验系数</summary>
		[ProtoMember(7)]
		public int LaborExpCoefficient { get; set; }
		/// <summary>伤疾恢复</summary>
		[ProtoMember(8)]
		public int DiseaseRecovery { get; set; }
		/// <summary>伤疾加重</summary>
		[ProtoMember(9)]
		public int DiseaseWorsen { get; set; }
		/// <summary>伤病减免</summary>
		[ProtoMember(10)]
		public int DiseaseRelief { get; set; }
		/// <summary>攻击加成</summary>
		[ProtoMember(11)]
		public int AttackBonusHPBonus { get; set; }
		/// <summary>生命加成</summary>
		[ProtoMember(12)]
		public int HPBonus { get; set; }
		/// <summary>防御加成</summary>
		[ProtoMember(13)]
		public int DEFBonus { get; set; }

	}
}
