using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class RegionConfigCategory : ProtoObject, IMerge
    {
        public static RegionConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, RegionConfig> dict = new Dictionary<int, RegionConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<RegionConfig> list = new List<RegionConfig>();
		
        public RegionConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            RegionConfigCategory s = o as RegionConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (RegionConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public RegionConfig Get(int id)
        {
            this.dict.TryGetValue(id, out RegionConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (RegionConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, RegionConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<RegionConfig> GetList()
        {
            return this.list;
        }

        public RegionConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class RegionConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>区域名称</summary>
		[ProtoMember(2)]
		public string Region { get; set; }
		/// <summary>描述</summary>
		[ProtoMember(3)]
		public string Description { get; set; }

	}
}
