using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class RoleConfigCategory : ProtoObject, IMerge
    {
        public static RoleConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, RoleConfig> dict = new Dictionary<int, RoleConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<RoleConfig> list = new List<RoleConfig>();
		
        public RoleConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            RoleConfigCategory s = o as RoleConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (RoleConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public RoleConfig Get(int id)
        {
            this.dict.TryGetValue(id, out RoleConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (RoleConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, RoleConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<RoleConfig> GetList()
        {
            return this.list;
        }

        public RoleConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class RoleConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>名称</summary>
		[ProtoMember(2)]
		public string Name { get; set; }
		/// <summary>类型</summary>
		[ProtoMember(3)]
		public int Type { get; set; }
		/// <summary>性别</summary>
		[ProtoMember(4)]
		public int Gender { get; set; }
		/// <summary>初始体质</summary>
		[ProtoMember(5)]
		public int Physique { get; set; }
		/// <summary>初始年龄</summary>
		[ProtoMember(6)]
		public int Age { get; set; }
		/// <summary>初始寿命上限</summary>
		[ProtoMember(7)]
		public int LifeMax { get; set; }
		/// <summary>劳力</summary>
		[ProtoMember(8)]
		public int Labor { get; set; }
		/// <summary>智慧</summary>
		[ProtoMember(9)]
		public int Wisdom { get; set; }
		/// <summary>移动速度</summary>
		[ProtoMember(10)]
		public int MoveSpeed { get; set; }
		/// <summary>属性</summary>
		[ProtoMember(11)]
		public int AttributesGroup { get; set; }

	}
}
