using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class UpgrateSequenceConfigCategory : ProtoObject, IMerge
    {
        public static UpgrateSequenceConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, UpgrateSequenceConfig> dict = new Dictionary<int, UpgrateSequenceConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<UpgrateSequenceConfig> list = new List<UpgrateSequenceConfig>();
		
        public UpgrateSequenceConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            UpgrateSequenceConfigCategory s = o as UpgrateSequenceConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (UpgrateSequenceConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public UpgrateSequenceConfig Get(int id)
        {
            this.dict.TryGetValue(id, out UpgrateSequenceConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (UpgrateSequenceConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, UpgrateSequenceConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<UpgrateSequenceConfig> GetList()
        {
            return this.list;
        }

        public UpgrateSequenceConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class UpgrateSequenceConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>序列类型</summary>
		[ProtoMember(2)]
		public int SequenceType { get; set; }
		/// <summary>序列物品编号</summary>
		[ProtoMember(3)]
		public int[] ItemId { get; set; }

	}
}
