using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using ET;
using UnityEditor;
using UnityEditor.SceneManagement;
using UnityEditor.UIElements;
using UnityEngine;
using UnityEngine.UIElements;
using Button = UnityEngine.UIElements.Button;

public class InitResourceEditorData
{
    public int ConfigId;
    public Transform ResourceRoot;
    public int MonsterGroupConfigId;
    // 刷新类型
    public int RefreshType;
    // 刷新参数
    public int RefreshParameter;

    public InitResourceEditorData(int configId, Transform root)
    {
        this.ConfigId = configId;
        this.ResourceRoot = root;
    }
}

public class MapUnitEditor : EditorWindow
{
    static float tileWidth = 1.0f;
    static float tileHeight = 0.5f;
    static float halfTileWidth = tileWidth / 2;
    static float halfTileHeight = tileHeight / 2;

    private static float tileMapWidth;
    private static float tileMapHeight;
    private static float halfTileMapWidth;
    private static float halfTileMapHeight;
    private static int MaxTileX;
    private static int MaxTileY;
    
    // for init resource
    static string resourceStr = "Resource";
    private static string initResourceEditRoot = "SceneRoot/MainSceneRoot/Scene/Layout/ResourceInitEditor";
    private Transform resourceEditRoot;
    private InitResourceEditorData curSelectData;
    private List<InitResourceEditorData> InitResourceEditorDataList = new List<InitResourceEditorData>();
    private Dictionary<GameObject, InitResourceEditorData> InitResourceEditorDataDict = new Dictionary<GameObject, InitResourceEditorData>();
    private ListView initResourceListView;
    private IntegerField monsterGroupConfigIdIF;
    private EnumField refreshTypeEF;
    private IntegerField refreshParameterIF;

    // for polygon collider
    private TextField colTF;
    private TextField rowTF;
    
    
    [MenuItem("Tools/MapUnitEditor")]
    public static void ShowMapUnitEditor()
    {
        MapUnitEditor wnd = GetWindow<MapUnitEditor>();
        wnd.titleContent = new GUIContent("MapUnitEditor");
    }

    public void CreateGUI()
    {
        // Each editor window contains a root VisualElement object
        VisualElement root = rootVisualElement;

        // Import UXML
        var visualTree = AssetDatabase.LoadAssetAtPath<VisualTreeAsset>("Assets/Editor/ResourceEditor/MapUnitEditor/MapUnitEditor.uxml");
        visualTree.CloneTree(root);

        // A stylesheet can be added to a VisualElement.
        // The style will be applied to the VisualElement and all of its children.
        var styleSheet = AssetDatabase.LoadAssetAtPath<StyleSheet>("Assets/Editor/ResourceEditor/MapUnitEditor/MapUnitEditor.uss");
        
        // for init resource
        Button adjustResourcePosBtn = root.Q<Button>("AdjustResourcePosBtn");
        adjustResourcePosBtn.clicked += this.OnAdjustResourcePosBtnClick;
        Button exportInitResourceDataBtn = root.Q<Button>("ExportInitResourceDataBtn");
        exportInitResourceDataBtn.clicked += this.OnExportIntResourceDataBtnClick;
        Button loadInitResourceDataBtn = root.Q<Button>("LoadInitResourceDataBtn");
        loadInitResourceDataBtn.clicked += this.OnLoadInitResourceDataBtnClick;

        this.initResourceListView = root.Q<ListView>("InitResourceListView");
        this.initResourceListView.bindItem = this.BindInitResourceListItem;
        this.initResourceListView.makeItem = this.MakeInitResourceListItem;
        this.initResourceListView.onSelectionChange += this.OnInitResourceItemChosen;

        this.monsterGroupConfigIdIF = root.Q<IntegerField>("MonsterGroupConfigIdIF");
        this.monsterGroupConfigIdIF.RegisterValueChangedCallback(OnMonsterGroupConfigIdChanged);

        refreshTypeEF = root.Q<EnumField>("RefreshTypeEF");
        refreshTypeEF.Init(MonsterGroupConfigRefreshTypeEnum.Disposable);
        this.refreshTypeEF.RegisterValueChangedCallback(OnRefreshTypeChanged);
        
        refreshParameterIF = root.Q<IntegerField>("RefreshParameterIF");
        this.refreshParameterIF.RegisterValueChangedCallback(OnRefreshParameterChanged);

        Selection.selectionChanged = OnGameObjectSelectionChanged;
        EditorApplication.hierarchyChanged += OnHierachyChanged;
        
        
        // for polygon collider
        this.rowTF = root.Q<TextField>("RowTF");
        this.colTF = root.Q<TextField>("ColTF");

        Button loadButton = root.Q<Button>("LoadPolygonBtn");
        loadButton.clicked += this.OnLoadButtonClick;

        Button genButton = root.Q<Button>("GenPolygonBtn");
        genButton.clicked += OnGenerateButtonClick;
    }

    private void OnHierachyChanged()
    {
        this.RefreshInitResourceListView();
    }

    private void RefreshInitResourceListView()
    {
        List<InitResourceEditorData> newInitResourceEditorDataList = new List<InitResourceEditorData>();
        if (this.resourceEditRoot == null)
        {
            return;
        }
        var childCount = resourceEditRoot.childCount;
        if (childCount > 0)
        {
            for (int i = 0; i < childCount; i++)
            {
                var child = resourceEditRoot.GetChild(i);
                var go = child.gameObject;
                if (this.InitResourceEditorDataDict.ContainsKey(go))
                {
                    newInitResourceEditorDataList.Add(this.InitResourceEditorDataDict[go]);
                }
                else
                {

                    var name = go.name;
                    int id = int.Parse(name.Split('_')[1]);
                    var editorData = new InitResourceEditorData(id, child);
                    newInitResourceEditorDataList.Add(editorData);
                }
            }
        }

        this.InitResourceEditorDataList = newInitResourceEditorDataList;
        this.InitResourceEditorDataDict.Clear();
        foreach (InitResourceEditorData initResourceEditorData in this.InitResourceEditorDataList)
        {
            this.InitResourceEditorDataDict.Add(initResourceEditorData.ResourceRoot.gameObject, initResourceEditorData);
        }
        
        this.initResourceListView.itemsSource = this.InitResourceEditorDataList;
        this.initResourceListView.RefreshItems();
    }

    private void OnGameObjectSelectionChanged()
    {
        var go = Selection.activeGameObject;
        if (go == null)
        {
            return;
        }
        for (int i = 0; i < InitResourceEditorDataList.Count; i++)
        {
            var data = this.InitResourceEditorDataList[i];
            if (data.ResourceRoot == go.transform)
            {
                this.initResourceListView.selectedIndex = i;
                break;
            }
        }
    }

    private void OnRefreshParameterChanged(ChangeEvent<int> evt)
    {
        if (this.curSelectData != null)
        {
            this.curSelectData.RefreshParameter = evt.newValue;
        }
    }

    private void OnRefreshTypeChanged(ChangeEvent<Enum> evt)
    {
        if (this.curSelectData != null)
        {
            this.curSelectData.RefreshType = Convert.ToInt32(evt.newValue);  
        }
    }

    private void OnMonsterGroupConfigIdChanged(ChangeEvent<int> evt)
    {
        if (this.curSelectData != null)
        {
            this.curSelectData.MonsterGroupConfigId = evt.newValue;  
        }
    }

    void BindInitResourceListItem(VisualElement ve, int index)
    {
        Label label = ve as Label;
        var data = this.InitResourceEditorDataList[index];
        label.text = data.ConfigId.ToString();
    }

    private VisualElement MakeInitResourceListItem()
    {
        var label = new Label();
        return label;
    }

    private void OnInitResourceItemChosen(IEnumerable<object> obj)
    {
        foreach (object o in obj)
        {
            var data = (InitResourceEditorData) o;
            curSelectData = data;
            this.monsterGroupConfigIdIF.value = data.MonsterGroupConfigId;
            this.refreshTypeEF.value = (MonsterGroupConfigRefreshTypeEnum)data.RefreshType;
            this.refreshParameterIF.value = data.RefreshParameter;
            Selection.activeGameObject = data.ResourceRoot.gameObject;
        }
    }

    private void OnLoadButtonClick()
    {
        GameObject sprObj = this.GetEditGameObject();

        if (sprObj == null)
        {
            return;
        }
        
        SpriteRenderer spriteRenderer = sprObj.GetComponent<SpriteRenderer>();
        if (spriteRenderer == null)
        {
            Debug.LogError("Can't find SpriteRenderer");
            return;
        }

        Vector2Int size = GetSizeBySpriteCollider(spriteRenderer);
        this.rowTF.value = size.x.ToString();
        this.colTF.value = size.y.ToString();
    }

    private void OnGenerateButtonClick()
    {
        int row = int.Parse(this.rowTF.value);
        int col = int.Parse(this.colTF.value);
        this.GeneratePolygonCollider(row, col);
    }

    public GameObject GetEditGameObject()
    {
        GameObject obj = Selection.activeGameObject;

        GameObject sprObj = obj;
        if (!obj.name.EndsWith("_Base"))
        {
            sprObj = obj.transform.Find("Resource_Base")?.gameObject;
            if (sprObj == null)
            {
                sprObj = obj.transform.Find("Build_Base")?.gameObject;
            }
        }

        return sprObj;
    }

    public void GeneratePolygonCollider(int row, int col)
    {
        GameObject sprObj = this.GetEditGameObject();

        PolygonCollider2D polygonCollider2D = sprObj.GetComponent<PolygonCollider2D>();
        if (polygonCollider2D == null)
        {
            polygonCollider2D = sprObj.AddComponent<PolygonCollider2D>();
        }

        var sizeX = row;
        var sizeY = col;

        Vector2 left = MapToLocalPos(sizeX, 0);
        Vector2 top = MapToLocalPos(sizeX, sizeY);
        Vector2 right = MapToLocalPos(0, sizeY);
        Vector2 bottom = Vector2.zero;

        polygonCollider2D.points = new[] { bottom, left, top, right };
        EditorUtility.SetDirty(sprObj);
    }
    
    public Vector2 MapToLocalPos(int row, int col)
    {
        Vector2 localPos = new Vector2();
        localPos.x = (col - row) * halfTileWidth;
        localPos.y = (row + col) * halfTileHeight;
        return localPos;
    }
    
    public Vector2Int LocalToMapPos(Vector2 topPos)
    {
        Vector2Int mapPos = new Vector2Int();
        int row = Mathf.RoundToInt(topPos.y / tileHeight - topPos.x / tileWidth);
        int col = Mathf.RoundToInt(topPos.y / tileHeight + topPos.x / tileWidth);
        mapPos.x = row;
        mapPos.y = col;
            
        return mapPos;
    }
    
    public Vector2Int GetSizeBySpriteCollider(SpriteRenderer spriteRenderer)
    {
            
        PolygonCollider2D polygonCollider2D = spriteRenderer.GetComponent<PolygonCollider2D>();
        if (polygonCollider2D == null)
        {
            Debug.LogWarning("Sprite don't has a polygoncollider");
            return Vector2Int.zero;
        }

        // 2 约定为多边形碰撞盒的顶点， 由工具生成
        Vector2Int size = LocalToMapPos(polygonCollider2D.points[2]);
        return size;
    }
    
    public static Vector2 MapToWorldPos(int row, int col)
    {
        Vector2 worldPos = new Vector2();
        worldPos.x = (col - row) * halfTileWidth;
        worldPos.y = -halfTileMapHeight + (row + col) * halfTileHeight;
        return worldPos;
    }

    public static Vector2Int WorldToMapPos(Vector2 worlPos)
    {
        Vector2Int mapPos = new Vector2Int();
        var offsetY = worlPos.y + halfTileMapHeight;
        mapPos.x = Mathf.RoundToInt(offsetY / tileHeight - worlPos.x / tileWidth);
        mapPos.y = Mathf.RoundToInt(worlPos.x / tileWidth + offsetY / tileHeight);

        mapPos.x = Mathf.Clamp(mapPos.x, 0, MaxTileX);
        mapPos.y = Mathf.Clamp(mapPos.y, 0, MaxTileY);
        return mapPos;
    }

    public void OnAdjustResourcePosBtnClick()
    {
        this.AdjustResourcePos();
    }

    private void AdjustResourcePos()
    {
        GameObject mapSizeObj = GameObject.Find("MapSize");
        if (mapSizeObj == null)
        {
            Debug.LogError("找不到MapSize节点");
            return;
        }
        Transform availablePlaceObj = mapSizeObj.transform.Find("AvailablePlaceSize");
        var collider2D = availablePlaceObj.GetComponent<BoxCollider2D>();
        var size = collider2D.size;
        
        float width = size.x;
        float height = width / 2;
        if (height > size.y)
        {
            height = size.y;
            width = height * 2;
        }

        tileMapWidth = width;
        tileMapHeight = height;
        halfTileMapWidth = tileMapWidth / 2;
        halfTileMapHeight = tileMapHeight / 2;
        MaxTileX = Mathf.FloorToInt(tileMapWidth / tileWidth);
        MaxTileY = MaxTileX;
        
        // 计算资源坐标
        var root = GameObject.Find(initResourceEditRoot).transform;
        var childCount = root.childCount;
        if (childCount > 0)
        {
            for (int i = 0; i < childCount; i++)
            {
                var child = root.GetChild(i);
                var position = child.position;

                Vector2Int mapPos = WorldToMapPos(position);
                var alignWorldPos = MapToWorldPos(mapPos.x, mapPos.y);
                alignWorldPos.x = (float)Math.Round(alignWorldPos.x, 2);
                alignWorldPos.y = (float)Math.Round(alignWorldPos.y, 2);
                child.localPosition = alignWorldPos;
            }
        }
    }

    public void OnExportIntResourceDataBtnClick()
    {
        // 先调整位置再导出
        AdjustResourcePos();
            
        var root = GameObject.Find(initResourceEditRoot)?.transform;

        if (root == null)
        {
            Debug.LogError($"找不到 {initResourceEditRoot} 节点");
            return;
        }

        if (InitResourceEditorDataList.Count <= 0)
        {
            return;
        }
        var initResource = new MainSceneInitResource();
        foreach (var initResourceEditorData in this.InitResourceEditorDataList)
        {
            int id = initResourceEditorData.ConfigId;
            var data = initResourceEditorData;
            var pos = data.ResourceRoot.transform.localPosition;
            
            float posX = pos.x;
            float posY = pos.y;
            Debug.Log($"id:[{id}],pos:[{posX},{posY}]");

            var item = new InitResourceItem(id, posX, posY);
            item.MonsterGroupConfigId = data.MonsterGroupConfigId;
            item.RefreshType = data.RefreshType;
            item.RefreshParameter = data.RefreshParameter;
            initResource.InitResourceItems.Add(item);
            
            DestroyImmediate(data.ResourceRoot.gameObject);
        }
        
        string toJson = JsonHelperMono.ToJson(initResource);

        // save to file
        string dirPath = Path.Join(Application.dataPath, "../../Config/Resource");
        if (!Directory.Exists(dirPath))
        {
            Directory.CreateDirectory(dirPath);
        }

        string filePath = Path.Join(dirPath, "InitResource.json");
        StreamWriter sw = new StreamWriter(filePath, false, Encoding.UTF8);
        sw.Write(toJson);
        sw.Close();

        EditorSceneManager.MarkSceneDirty(EditorSceneManager.GetActiveScene());
        EditorSceneManager.SaveScene(EditorSceneManager.GetActiveScene());
        
        this.CopyInitResourceCSFile();
    }

    public void CopyInitResourceCSFile()
    {
        string sourceFile = Path.Join(Application.dataPath, "Editor/ResourceEditor/MapUnitEditor/MainSceneInitResource.cs");
        string distPath = Path.Join(Application.dataPath, "../../Server/Model/Demo/Resource/MainSceneInitResource.cs");
        
        FileInfo file = new(sourceFile);
        if (file.Exists)
        {
            file.CopyTo(distPath, true);
        }
    }

    public void OnLoadInitResourceDataBtnClick()
    {
        var root = GameObject.Find(initResourceEditRoot).transform;

        string filePath = Path.Join(Application.dataPath, "../../Config/Resource/InitResource.json");

        if (!File.Exists(filePath))
        {
            Debug.LogError($"{filePath} not found");
            return;
        }

        StreamReader sr = new StreamReader(filePath);

        var data = sr.ReadToEnd();

        MainSceneInitResource ir = JsonHelperMono.FromJson<MainSceneInitResource>(data);

        var childCount = root.childCount;
        for (int i = childCount - 1; i > 0; i--)
        {
            var child = root.GetChild(i);
            DestroyImmediate(child.gameObject);
        }

        this.InitResourceEditorDataList.Clear();
        string resourceDirPath = "Assets/Bundles/Resource";
        foreach (var item in ir.InitResourceItems)
        {
            string resourceName = resourceStr + "_" + item.ConfigId;
            string resourcePath = Path.Combine(resourceDirPath, resourceName + ".prefab");
            var prefab = AssetDatabase.LoadAssetAtPath<GameObject>(resourcePath);
            GameObject resGO = PrefabUtility.InstantiatePrefab(prefab) as GameObject;
            resGO.transform.parent = root;
            resGO.transform.localPosition = new Vector3(item.PosX, item.PosY, 0);
            resGO.transform.localScale = Vector3.one;

            var editorData = new InitResourceEditorData(item.ConfigId, resGO.transform);
            editorData.MonsterGroupConfigId = item.MonsterGroupConfigId;
            editorData.RefreshType = item.RefreshType;
            editorData.RefreshParameter = item.RefreshParameter;
            this.InitResourceEditorDataList.Add(editorData);
            InitResourceEditorDataDict.Add(resGO, editorData);
        }

        this.initResourceListView.itemsSource = this.InitResourceEditorDataList;
        this.initResourceListView.RefreshItems();

        resourceEditRoot = GameObject.Find(initResourceEditRoot).transform;

        sr.Close();
    }
}