using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class DialogueGroupConfigCategory : ProtoObject, IMerge
    {
        public static DialogueGroupConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, DialogueGroupConfig> dict = new Dictionary<int, DialogueGroupConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<DialogueGroupConfig> list = new List<DialogueGroupConfig>();
		
        public DialogueGroupConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            DialogueGroupConfigCategory s = o as DialogueGroupConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (DialogueGroupConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public DialogueGroupConfig Get(int id)
        {
            this.dict.TryGetValue(id, out DialogueGroupConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (DialogueGroupConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, DialogueGroupConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<DialogueGroupConfig> GetList()
        {
            return this.list;
        }

        public DialogueGroupConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class DialogueGroupConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>对白编号</summary>
		[ProtoMember(2)]
		public int[] DialogueId { get; set; }
		/// <summary>对话对象类型</summary>
		[ProtoMember(3)]
		public int DialogueNumber { get; set; }
		/// <summary>对白顺序</summary>
		[ProtoMember(4)]
		public int[] DialogueOrder { get; set; }

	}
}
