using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class DiseaseConfigCategory : ProtoObject, IMerge
    {
        public static DiseaseConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, DiseaseConfig> dict = new Dictionary<int, DiseaseConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<DiseaseConfig> list = new List<DiseaseConfig>();
		
        public DiseaseConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            DiseaseConfigCategory s = o as DiseaseConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (DiseaseConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public DiseaseConfig Get(int id)
        {
            this.dict.TryGetValue(id, out DiseaseConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (DiseaseConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, DiseaseConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<DiseaseConfig> GetList()
        {
            return this.list;
        }

        public DiseaseConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class DiseaseConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>疾病名称</summary>
		[ProtoMember(2)]
		public string Name { get; set; }
		/// <summary>类型</summary>
		[ProtoMember(3)]
		public int Type { get; set; }
		/// <summary>等级</summary>
		[ProtoMember(4)]
		public int Level { get; set; }
		/// <summary>程度最小值</summary>
		[ProtoMember(5)]
		public int Min { get; set; }
		/// <summary>程度最大值</summary>
		[ProtoMember(6)]
		public int Max { get; set; }

	}
}
