using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class EquipmentConfigCategory : ProtoObject, IMerge
    {
        public static EquipmentConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, EquipmentConfig> dict = new Dictionary<int, EquipmentConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<EquipmentConfig> list = new List<EquipmentConfig>();
		
        public EquipmentConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            EquipmentConfigCategory s = o as EquipmentConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (EquipmentConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public EquipmentConfig Get(int id)
        {
            this.dict.TryGetValue(id, out EquipmentConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (EquipmentConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, EquipmentConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<EquipmentConfig> GetList()
        {
            return this.list;
        }

        public EquipmentConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class EquipmentConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>名称</summary>
		[ProtoMember(2)]
		public string Name { get; set; }
		/// <summary>类型</summary>
		[ProtoMember(3)]
		public int Type { get; set; }
		/// <summary>作用对象</summary>
		[ProtoMember(4)]
		public int ActionObject { get; set; }
		/// <summary>效果类型</summary>
		[ProtoMember(5)]
		public int EffectType { get; set; }
		/// <summary>效果值</summary>
		[ProtoMember(6)]
		public int EffectValue { get; set; }

	}
}
