using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class EventGroupConfigCategory : ProtoObject, IMerge
    {
        public static EventGroupConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, EventGroupConfig> dict = new Dictionary<int, EventGroupConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<EventGroupConfig> list = new List<EventGroupConfig>();
		
        public EventGroupConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            EventGroupConfigCategory s = o as EventGroupConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (EventGroupConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public EventGroupConfig Get(int id)
        {
            this.dict.TryGetValue(id, out EventGroupConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (EventGroupConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, EventGroupConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<EventGroupConfig> GetList()
        {
            return this.list;
        }

        public EventGroupConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class EventGroupConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>类型</summary>
		[ProtoMember(2)]
		public int Type { get; set; }
		/// <summary>类型参数</summary>
		[ProtoMember(3)]
		public int TypeParameter { get; set; }
		/// <summary>触发条件</summary>
		[ProtoMember(4)]
		public int[] TriggerCondition { get; set; }
		/// <summary>条件参数</summary>
		[ProtoMember(5)]
		public int[] ConditionParameter { get; set; }
		/// <summary>事件编号</summary>
		[ProtoMember(6)]
		public int[] Event { get; set; }
		/// <summary>权重</summary>
		[ProtoMember(7)]
		public int[] EventWeight { get; set; }
		/// <summary>叠加权重</summary>
		[ProtoMember(8)]
		public int[] OverlayWeight { get; set; }
		/// <summary>事件组CD</summary>
		[ProtoMember(9)]
		public int CD { get; set; }

	}
}
