using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class ForestConfigCategory : ProtoObject, IMerge
    {
        public static ForestConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, ForestConfig> dict = new Dictionary<int, ForestConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<ForestConfig> list = new List<ForestConfig>();
		
        public ForestConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            ForestConfigCategory s = o as ForestConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (ForestConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public ForestConfig Get(int id)
        {
            this.dict.TryGetValue(id, out ForestConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (ForestConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, ForestConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<ForestConfig> GetList()
        {
            return this.list;
        }

        public ForestConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class ForestConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>名称</summary>
		[ProtoMember(2)]
		public string Name { get; set; }
		/// <summary>危险程度</summary>
		[ProtoMember(3)]
		public int Dangrous { get; set; }
		/// <summary>珍稀资源</summary>
		[ProtoMember(4)]
		public int RareResources { get; set; }
		/// <summary>资源数量</summary>
		[ProtoMember(5)]
		public int ResourcesNumber { get; set; }
		/// <summary>资源点</summary>
		[ProtoMember(6)]
		public int[] ResourcesSetting { get; set; }
		/// <summary>怪物点</summary>
		[ProtoMember(7)]
		public int[] MonstersSetting { get; set; }
		/// <summary>战力要求</summary>
		[ProtoMember(8)]
		public int Requirements { get; set; }
		/// <summary>描述</summary>
		[ProtoMember(9)]
		public string Description { get; set; }

	}
}
