using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class MaterialConfigCategory : ProtoObject, IMerge
    {
        public static MaterialConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, MaterialConfig> dict = new Dictionary<int, MaterialConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<MaterialConfig> list = new List<MaterialConfig>();
		
        public MaterialConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            MaterialConfigCategory s = o as MaterialConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (MaterialConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public MaterialConfig Get(int id)
        {
            this.dict.TryGetValue(id, out MaterialConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (MaterialConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, MaterialConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<MaterialConfig> GetList()
        {
            return this.list;
        }

        public MaterialConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class MaterialConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>名称</summary>
		[ProtoMember(2)]
		public string Name { get; set; }
		/// <summary>类型</summary>
		[ProtoMember(3)]
		public int[] Type { get; set; }
		/// <summary>可转化食物数量</summary>
		[ProtoMember(4)]
		public int FoodNum { get; set; }
		/// <summary>可转化水数量</summary>
		[ProtoMember(5)]
		public int WaterNum { get; set; }

	}
}
