using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class ResourcesPointConfigCategory : ProtoObject, IMerge
    {
        public static ResourcesPointConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, ResourcesPointConfig> dict = new Dictionary<int, ResourcesPointConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<ResourcesPointConfig> list = new List<ResourcesPointConfig>();
		
        public ResourcesPointConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            ResourcesPointConfigCategory s = o as ResourcesPointConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (ResourcesPointConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public ResourcesPointConfig Get(int id)
        {
            this.dict.TryGetValue(id, out ResourcesPointConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (ResourcesPointConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, ResourcesPointConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<ResourcesPointConfig> GetList()
        {
            return this.list;
        }

        public ResourcesPointConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class ResourcesPointConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>资源点名称</summary>
		[ProtoMember(2)]
		public string ResourcesPointName { get; set; }
		/// <summary>类型</summary>
		[ProtoMember(3)]
		public int Type { get; set; }
		/// <summary>劳作主体</summary>
		[ProtoMember(4)]
		public int[] WorkerType { get; set; }
		/// <summary>关联资源</summary>
		[ProtoMember(5)]
		public int[] RelatedResources { get; set; }
		/// <summary>预置文件</summary>
		[ProtoMember(6)]
		public string[] Prefab { get; set; }
		/// <summary>资源图片</summary>
		[ProtoMember(7)]
		public string[] ResourcePicture { get; set; }
		/// <summary>月圆夜预置文件</summary>
		[ProtoMember(8)]
		public string MoonnightPrefab { get; set; }
		/// <summary>月圆夜图片</summary>
		[ProtoMember(9)]
		public string MoonnightPicture { get; set; }
		/// <summary>可否铲除</summary>
		[ProtoMember(10)]
		public int RootOut { get; set; }
		/// <summary>幼苗编号</summary>
		[ProtoMember(11)]
		public int SaplingItemId { get; set; }
		/// <summary>详情描述</summary>
		[ProtoMember(12)]
		public string Description { get; set; }

	}
}
