using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class RewardGroupConfigCategory : ProtoObject, IMerge
    {
        public static RewardGroupConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, RewardGroupConfig> dict = new Dictionary<int, RewardGroupConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<RewardGroupConfig> list = new List<RewardGroupConfig>();
		
        public RewardGroupConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            RewardGroupConfigCategory s = o as RewardGroupConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (RewardGroupConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public RewardGroupConfig Get(int id)
        {
            this.dict.TryGetValue(id, out RewardGroupConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (RewardGroupConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, RewardGroupConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<RewardGroupConfig> GetList()
        {
            return this.list;
        }

        public RewardGroupConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class RewardGroupConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>物品编号</summary>
		[ProtoMember(2)]
		public int[] ItemId { get; set; }
		/// <summary>数量</summary>
		[ProtoMember(3)]
		public int[] Number { get; set; }

	}
}
