using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class SaplingConfigCategory : ProtoObject, IMerge
    {
        public static SaplingConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, SaplingConfig> dict = new Dictionary<int, SaplingConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<SaplingConfig> list = new List<SaplingConfig>();
		
        public SaplingConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            SaplingConfigCategory s = o as SaplingConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (SaplingConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public SaplingConfig Get(int id)
        {
            this.dict.TryGetValue(id, out SaplingConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (SaplingConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, SaplingConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<SaplingConfig> GetList()
        {
            return this.list;
        }

        public SaplingConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class SaplingConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>幼苗名称</summary>
		[ProtoMember(2)]
		public string SaplingName { get; set; }
		/// <summary>成长时间</summary>
		[ProtoMember(3)]
		public int GrowthTime { get; set; }
		/// <summary>关联资源点</summary>
		[ProtoMember(4)]
		public int RelatedResourcesPoint { get; set; }
		/// <summary>预置体</summary>
		[ProtoMember(5)]
		public string SaplingPrefab { get; set; }
		/// <summary>描述</summary>
		[ProtoMember(6)]
		public string Description { get; set; }

	}
}
