using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class SeasonConfigCategory : ProtoObject, IMerge
    {
        public static SeasonConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, SeasonConfig> dict = new Dictionary<int, SeasonConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<SeasonConfig> list = new List<SeasonConfig>();
		
        public SeasonConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            SeasonConfigCategory s = o as SeasonConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (SeasonConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public SeasonConfig Get(int id)
        {
            this.dict.TryGetValue(id, out SeasonConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (SeasonConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, SeasonConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<SeasonConfig> GetList()
        {
            return this.list;
        }

        public SeasonConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class SeasonConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>季节名称</summary>
		[ProtoMember(2)]
		public string Name { get; set; }
		/// <summary>季节类型</summary>
		[ProtoMember(3)]
		public int Tapy { get; set; }
		/// <summary>游戏阶段</summary>
		[ProtoMember(4)]
		public int Stage { get; set; }
		/// <summary>每日起止时间</summary>
		[ProtoMember(5)]
		public int[] DayTime { get; set; }
		/// <summary>季节天气组</summary>
		[ProtoMember(6)]
		public int[] WeatherGroup { get; set; }
		/// <summary>天气权重</summary>
		[ProtoMember(7)]
		public int[] Weights { get; set; }
		/// <summary>季节描述</summary>
		[ProtoMember(8)]
		public string Description { get; set; }
		/// <summary>季节图片</summary>
		[ProtoMember(9)]
		public string Picture { get; set; }

	}
}
