using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class SkillExpConfigCategory : ProtoObject, IMerge
    {
        public static SkillExpConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, SkillExpConfig> dict = new Dictionary<int, SkillExpConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<SkillExpConfig> list = new List<SkillExpConfig>();
		
        public SkillExpConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            SkillExpConfigCategory s = o as SkillExpConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (SkillExpConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public SkillExpConfig Get(int id)
        {
            this.dict.TryGetValue(id, out SkillExpConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (SkillExpConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, SkillExpConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<SkillExpConfig> GetList()
        {
            return this.list;
        }

        public SkillExpConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class SkillExpConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>等级</summary>
		[ProtoMember(2)]
		public int Lv { get; set; }
		/// <summary>升级修为</summary>
		[ProtoMember(3)]
		public int Sophisticate { get; set; }
		/// <summary>升级耗时</summary>
		[ProtoMember(4)]
		public int TimeConsum { get; set; }

	}
}
