using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class SkillSetConfigCategory : ProtoObject, IMerge
    {
        public static SkillSetConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, SkillSetConfig> dict = new Dictionary<int, SkillSetConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<SkillSetConfig> list = new List<SkillSetConfig>();
		
        public SkillSetConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            SkillSetConfigCategory s = o as SkillSetConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (SkillSetConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public SkillSetConfig Get(int id)
        {
            this.dict.TryGetValue(id, out SkillSetConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (SkillSetConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, SkillSetConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<SkillSetConfig> GetList()
        {
            return this.list;
        }

        public SkillSetConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class SkillSetConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>武器限制</summary>
		[ProtoMember(2)]
		public int Weapon { get; set; }
		/// <summary>类型</summary>
		[ProtoMember(3)]
		public int Type { get; set; }
		/// <summary>一阶技能</summary>
		[ProtoMember(4)]
		public int RankOne { get; set; }
		/// <summary>二阶技能</summary>
		[ProtoMember(5)]
		public int RankTwo { get; set; }
		/// <summary>三阶技能</summary>
		[ProtoMember(6)]
		public int RankThree { get; set; }
		/// <summary>四阶技能</summary>
		[ProtoMember(7)]
		public int RankFour { get; set; }
		/// <summary>终结技</summary>
		[ProtoMember(8)]
		public int RankMax { get; set; }
		/// <summary>终结技冷却</summary>
		[ProtoMember(9)]
		public int RankMaxCD { get; set; }
		/// <summary>终结技消耗</summary>
		[ProtoMember(10)]
		public int RankMaxCost { get; set; }

	}
}
