using System;
using System.Collections.Generic;
using MongoDB.Bson.Serialization.Attributes;
using ProtoBuf;

namespace ET
{
    [ProtoContract]
    [Config]
    public partial class SpecialItemConfigCategory : ProtoObject, IMerge
    {
        public static SpecialItemConfigCategory Instance;
		
        [ProtoIgnore]
        [BsonIgnore]
        private Dictionary<int, SpecialItemConfig> dict = new Dictionary<int, SpecialItemConfig>();
		
        [BsonElement]
        [ProtoMember(1)]
        private List<SpecialItemConfig> list = new List<SpecialItemConfig>();
		
        public SpecialItemConfigCategory()
        {
            Instance = this;
        }
        
        public void Merge(object o)
        {
            SpecialItemConfigCategory s = o as SpecialItemConfigCategory;
            this.list.AddRange(s.list);
        }
		
        public override void EndInit()
        {
            foreach (SpecialItemConfig config in list)
            {
                config.EndInit();
                this.dict.Add(config.Id, config);
            }            
            this.AfterEndInit();
        }
		
        public SpecialItemConfig Get(int id)
        {
            this.dict.TryGetValue(id, out SpecialItemConfig item);

            if (item == null)
            {
                throw new Exception($"配置找不到，配置表名: {nameof (SpecialItemConfig)}，配置id: {id}");
            }

            return item;
        }
		
        public bool Contain(int id)
        {
            return this.dict.ContainsKey(id);
        }

        public Dictionary<int, SpecialItemConfig> GetAll()
        {
            return this.dict;
        }
        
        public List<SpecialItemConfig> GetList()
        {
            return this.list;
        }

        public SpecialItemConfig GetOne()
        {
            if (this.dict == null || this.dict.Count <= 0)
            {
                return null;
            }
            return this.dict.Values.GetEnumerator().Current;
        }
    }

    [ProtoContract]
	public partial class SpecialItemConfig: ProtoObject, IConfig
	{
		/// <summary>编号</summary>
		[ProtoMember(1)]
		public int Id { get; set; }
		/// <summary>名称</summary>
		[ProtoMember(2)]
		public string Name { get; set; }
		/// <summary>事件进度</summary>
		[ProtoMember(3)]
		public int[] EventProgress { get; set; }
		/// <summary>获得物品</summary>
		[ProtoMember(4)]
		public int[] GetItem { get; set; }
		/// <summary>动作动画</summary>
		[ProtoMember(5)]
		public string Animation { get; set; }

	}
}
